//const int __gba_multiboot;

#include "string.h"
#include "varargs.h"
#include "debug.c"

#include "header\GBA.h"
#include "header\BG.h"
#include "header\Timer.h"
#include "header\Blend.h"
#include "header\sprite.h"

#include "pcmstream.h"

#define attriwram __attribute__ ((section (".data.iwram")))
//#define attriwram 

void SRAM_SaveIWRAM(void);

#include "interrupt.c"
#include "boolean.h"
#include "GBATools.c"

u8 *PluginFilename;
u8 *TextFilename;
u8 *TextData;
u32 TextDataSize;

#include "textcnt.c"
#include "main_img.c"
#include "fontpack.c"
#include "OAMTools.c"

#include "pogoshell2_args.c"
#include "pogoshell2_rtc.c"

#define Color_FlameClear 0
#define Color_FlameBG 1
#define Color_FlameBright 8
#define Color_FlameMask 9

#include "sramptxv.c"
#include "sndeff.c"
#include "menu.c"

extern u8 __iwram_overlay_lma;

u32 LineHeight;
u32 LineCount;
u32 LinePos;
u32 LineSpace;
u32 LineMaxLength;
s32 LineDrawOffset;
u32 KeyRepeatTime;

void BGInit(void)
{
  // [hݒ
  REG_DISPCNT=MODE_1;
  
  REG_BG0CNT = ( BG_SIZEA_256_256 | BG_COLOR_16 | BG_CHARBASE(ADR_BGCHR0) | BG_MAPBASE(ADR_BGMAP0) | 1);
  REG_BG1CNT = ( BG_SIZEA_256_256 | BG_COLOR_16 | BG_CHARBASE(ADR_BGCHR1) | BG_MAPBASE(ADR_BGMAP1) | 0);
  REG_BG2CNT = 0;
  REG_BG3CNT = 0;
  
  REG_BG0HOFS=0;
  REG_BG0VOFS=0;
  REG_BG1HOFS=0;
  REG_BG1VOFS=0;
  REG_BG2HOFS=0;
  REG_BG2VOFS=0;
  REG_BG3HOFS=0;
  REG_BG3VOFS=0;

  u16 GrayRGB(u16 b)
  {
    return(RGB(b+1,b,b));
  }
  
  BGPAL[0x00+0]=GrayRGB(0x00);
  BGPAL[0x00+1]=GrayRGB(0x1e);
  BGPAL[0x00+2]=GrayRGB(0x17);
  BGPAL[0x00+3]=GrayRGB(0x11);
  BGPAL[0x00+4]=GrayRGB(0x0b);
  BGPAL[0x00+5]=GrayRGB(0x05);
  BGPAL[0x00+6]=GrayRGB(0x00);
  
  BGPAL[0x10+0]=GrayRGB(0x00);
  BGPAL[0x10+1]=GrayRGB(0x00);
  BGPAL[0x10+2]=GrayRGB(0x05);
  BGPAL[0x10+3]=GrayRGB(0x0b);
  BGPAL[0x10+4]=GrayRGB(0x11);
  BGPAL[0x10+5]=GrayRGB(0x17);
  BGPAL[0x10+6]=GrayRGB(0x1e);
  
  BGPAL[0x10+Color_FlameBright]=RGB(0x0e,0x0d,0x0d);
  BGPAL[0x10+Color_FlameMask]=RGB(0x15,0x13,0x13);
  
  u16 tile;
  u16 *MAP0=&BGMAP0[0];
  u16 *MAP1=&BGMAP1[0];
  for(tile=0;tile<(256/8)*(256/8);tile++){
    *MAP0=(0<<12)+tile;
    *MAP1=(1<<12)+tile;
    MAP0++;
    MAP1++;
  }
  
  REG_DISPCNT=MODE_1 | BG0_ENABLE | BG1_ENABLE;
}

b8 VBlank_Enabled;
s32 VBlank_Position;
u32 VBlank_Volume;

attriwram void IRQ_VBlank_Dummy(void)
{
}

attriwram void IRQ_VBlank_UpScroll(void)
{
  if(VBlank_Volume!=1){
    VBlank_Position-=2;
    VBlank_Volume-=2;
    }else{
    VBlank_Position-=1;
    VBlank_Volume-=1;
  }
  REG_BG0VOFS=(LineHeight*2)+LineDrawOffset+VBlank_Position;
  if(VBlank_Volume==0){
    IRQTable[IRQIndex_VBlank]=(u32)IRQ_VBlank_Dummy;
    VBlank_Enabled=False;
  }
}

attriwram void IRQ_VBlank_DownScroll(void)
{
  if(VBlank_Volume!=1){
    VBlank_Position+=2;
    VBlank_Volume-=2;
    }else{
    VBlank_Position+=1;
    VBlank_Volume-=1;
  }
  REG_BG0VOFS=(LineHeight*2)+LineDrawOffset+VBlank_Position;
  if(VBlank_Volume==0){
    IRQTable[IRQIndex_VBlank]=(u32)IRQ_VBlank_Dummy;
    VBlank_Enabled=False;
  }
}

void TextScroll(s32 Vector,b8 Smooth)
{
  if(Vector==-1){
    if(LinePos==0) return;
  }
  if(Vector==1){
    if(TextRealizeLine<=(LinePos+1)) return;
  }
  LinePos+=Vector;
  
  if(Smooth==False){
    VBlank_Enabled=False;
    }else{
    VBlank_Enabled=True;
    VBlank_Position=0;
    VBlank_Volume=LineHeight;
    if(Vector==-1){
      IRQTable[IRQIndex_VBlank]=(u32)IRQ_VBlank_UpScroll;
      }else{
      IRQTable[IRQIndex_VBlank]=(u32)IRQ_VBlank_DownScroll;
    }
  }
  
  u32 ofs,drawy;
  if(Vector==-1){
    ofs=Text_GetLinePosition(LinePos,LineMaxLength);
    drawy=LineHeight+LineDrawOffset;
    }else{
    ofs=Text_GetLinePosition(LinePos+LineCount-1,LineMaxLength);
    drawy=(LineCount-1+3)*LineHeight+LineDrawOffset;
  }
  if(ofs!=-1){
    FontData_BGCHRDrawText(BGCHR0,0,drawy,(u8*)&TextData[ofs]);
    }else{
    FontData_BGCHRDrawText(BGCHR0,0,drawy,"");
  }
  if(LineSpace!=0){
    FontData_BGCHRFill(BGCHR0,0,drawy+LineHeight-LineSpace,240,LineSpace,1);
  }
  
  if(Smooth==False){
    }else{
    while(VBlank_Enabled==True);
    IRQTable[IRQIndex_VBlank]=(u32)IRQ_VBlank_Dummy;
  }
  
  if(Vector==-1){
    LineDrawOffset-=LineHeight;
    }else{
    LineDrawOffset+=LineHeight;
  }

  u32 BaseAdr=(u32)&BGCHR0[0];
  u32 size=((256-32)/8*32)*32;
  
  REG_DM3CNT_H = DMA_TRANSFER_OFF | DMA_INTR_OFF | DMA_TIMING_NOW | DMA_SIZE_16 | DMA_REPEAT_OFF;
  REG_DM3CNT_L = size/2; // 16bit
  if(LineDrawOffset<0){
    REG_DM3SAD = BaseAdr+size;
    REG_DM3DAD = BaseAdr+(((-LineDrawOffset)/8*32)*32)+size;
    REG_DM3CNT_H |= DMA_TRANSFER_ON | DMA_SAD_DEC | DMA_DAD_DEC;
    LineDrawOffset+=(-LineDrawOffset/8)*8;
    }else{
    REG_DM3SAD = BaseAdr+((LineDrawOffset/8*32)*32);
    REG_DM3DAD = BaseAdr;
    REG_DM3CNT_H |= DMA_TRANSFER_ON | DMA_SAD_INC | DMA_DAD_INC;
    LineDrawOffset-=(LineDrawOffset/8)*8;
  }
  while((REG_DM3CNT_H&DMA_TRANSFER_ON)!=0);
  REG_DM3CNT_H = DMA_TRANSFER_OFF;
  REG_BG0VOFS=(LineHeight*2)+LineDrawOffset;
}

void PageScroll(s32 Vector)
{
  u32 cnt;
  if(Vector<0){
    for(cnt=0;cnt<-Vector;cnt++){
      TextScroll(-1,False);
    }
    }else{
    for(cnt=0;cnt<Vector;cnt++){
      TextScroll(1,False);
    }
  }
}

void PageRedraw(void)
{
  u32 ty;
  u32 ofs;
  for(ty=0;ty<LineCount;ty++){
    ofs=Text_GetLinePosition(LinePos+ty,LineMaxLength);
    if(ofs!=-1){
      FontData_BGCHRDrawText(BGCHR0,0,(ty+2)*LineHeight+LineDrawOffset,(u8*)&TextData[ofs]);
      }else{
      FontData_BGCHRDrawText(BGCHR0,0,(ty+2)*LineHeight+LineDrawOffset,"");
    }
    if(LineSpace!=0){
      FontData_BGCHRFill(BGCHR0,0,(ty+2)*LineHeight+LineDrawOffset+LineHeight-LineSpace,240,LineSpace,1);
    }
  }
}

u32 LoadLastLine(void)
{
  return(Text_SeekByteToLine(SRAM_Read32(&SRAM_CurrentState->LastPosition),LineMaxLength));
}

void SaveLastLine(u32 LinePos)
{
  SRAM_Write32(&SRAM_CurrentState->LastPosition,Text_GetLinePosition(LinePos,LineMaxLength));
}

void ChangeFontPack(u32 idx)
{
  FontPack_Select(idx);
  LineSpace=SRAM_Read32(&SRAM->LineSpace);
  LineHeight=FontPack->FontHeight+LineSpace;
  LineCount=160/LineHeight+1;
  LineDrawOffset=0;
  LineMaxLength=240/FontPack->FontHeight*2-1;
  Text_InitTextPageCache();
  REG_BG0VOFS=(LineHeight*2)+LineDrawOffset;
}

u8 ClockStrCache[14];

void Status_HidePanel(void)
{
  FontData_BGCHRFill(BGCHR1,0,160-11,240,1,0);
  FontData_BGCHRFill(BGCHR1,0,160-10,240,10,0);
}

void Status_ShowPanel(void)
{
  FontData_BGCHRFill(BGCHR1,0,160-11,240,1,9);
  FontData_BGCHRFill(BGCHR1,0,160-10,240,10,1);
  
  BG1_putpixel4(0,160-10-1,0x0000);
  BG1_putpixel4(0,160-10+0,0x8900);
  BG1_putpixel4(0,160-10+1,0x1190);
  BG1_putpixel4(0,160-10+2,0x1119);
  BG1_putpixel4(0,160-10+3,0x1119);
  
  BG1_putpixel4(240-4,160-10-1,0x0000);
  BG1_putpixel4(240-4,160-10+0,0x0098);
  BG1_putpixel4(240-4,160-10+1,0x0911);
  BG1_putpixel4(240-4,160-10+2,0x9111);
  BG1_putpixel4(240-4,160-10+3,0x9111);
  
  u32 cnt;
  for(cnt=0;cnt<14;cnt++){
    ClockStrCache[cnt]=0;
  }
}

void Status_LinePos(void)
{
  TFontPack *BackupFontPack=FontPack;
  FontPack_Select(0);
  
  inttostr(LinePos,5);
  FontData_BGCHRDrawText(BGCHR1,1*5,160-FontPack->FontHeight,res_inttostr);
  FontData_BGCHRDrawText(BGCHR1,(1+5)*5,160-FontPack->FontHeight,"line");
  
  FontPack=BackupFontPack;
}

void Status_BytePos(void)
{
  TFontPack *BackupFontPack=FontPack;
  FontPack_Select(0);
  
  inttostr(Text_GetLinePosition(LinePos,LineMaxLength),7);
  FontData_BGCHRDrawText(BGCHR1,11*5,160-FontPack->FontHeight,res_inttostr);
  FontData_BGCHRDrawText(BGCHR1,(11+7)*5,160-FontPack->FontHeight,"/");
  
  FontPack=BackupFontPack;
}

void Status_ByteSize(void)
{
  TFontPack *BackupFontPack=FontPack;
  FontPack_Select(0);
  
  inttostr(TextDataSize,7);
  FontData_BGCHRDrawText(BGCHR1,(11+8)*5,160-FontPack->FontHeight,res_inttostr);
  FontData_BGCHRDrawText(BGCHR1,(11+8+7)*5,160-FontPack->FontHeight,"byte");
  
  FontPack=BackupFontPack;
}

extern u32 pogoshell_gettime(void);

void Status_Clock(void)
{
  u8 str[14];
  
/*
  u32 t;
  t=pogones_gettime();
  uprintf("t=%x\n",t);
  FontData_BGCHRDrawText(BGCHR1,30,30,resprintf);
*/
  
  if(rtc_check()!=0x40){
    str[ 0]=' ';
    str[ 1]='R';
    str[ 2]='T';
    str[ 3]='C';
    str[ 4]=' ';
    str[ 5]='D';
    str[ 6]='i';
    str[ 7]='s';
    str[ 8]='a';
    str[ 9]='b';
    str[10]='l';
    str[11]='e';
    str[12]='d';
    str[13]='.';
    }else{
    TTimeFormat now;
    time2(&now);
    
    if(now.mon<10){
      str[0]=' ';
      }else{
      str[0]='0'+(now.mon/10);
    }
    str[1]='0'+(now.mon%10);
    str[2]='/';
    
    if(now.mday<10){
      str[3]=' ';
      }else{
      str[3]='0'+(now.mday/10);
    }
    str[4]='0'+(now.mday%10);
    str[5]=' ';
    
    if(now.hour<10){
      str[6]=' ';
      }else{
      str[6]='0'+(now.hour/10);
    }
    str[7]='0'+(now.hour%10);
    str[8]=':';
    
    str[9]='0'+(now.min/10);
    str[10]='0'+(now.min%10);
    str[11]=':';
    str[12]='0'+(now.sec/10);
    str[13]='0'+(now.sec%10);
  }
  
  TFontPack *BackupFontPack=FontPack;
  FontPack_Select(0);
  
  u32 bx,by;
  bx=32*5;
  by=160-FontPack->FontHeight;
  
  TFontData *FontData;
  b8 chg=False;
  u32 cnt,c1;
  for(cnt=0;cnt<14;cnt++){
    c1=str[cnt];
    if(ClockStrCache[cnt]!=c1){
      chg=True;
    }
    if(chg==True){
      ClockStrCache[cnt]=c1;
      FontData=FontData_FindFont(c1,0x00);
      FontData_Decode(FontData);
      FontData_BGCHRDrawDecodeAnk(BGCHR1,bx,by);
    }
    bx+=5;
  }
  
  FontPack=BackupFontPack;
}

void Status_All(void)
{
  Status_ShowPanel();
  Status_LinePos();
  Status_BytePos();
  Status_ByteSize();
  Status_Clock();
}

/*
  REG_TM0CNT = 0;
  REG_TM0D=0;
  REG_TM0CNT = TM_FREQ_PER_1024 | TM_ENABLE;
  uprintf("%d/100ms",((u32)REG_TM0D)*100000/65517);
  FontData_BGCHRDrawText(BGCHR1,0,4*16,resprintf);
  dprint(resprintf);
*/

void _ReturnPogoShell2(void)
{
  SaveLastLine(LinePos);
  Status_HidePanel();
  REG_BLDMOD=BLEND_TOP_BG0 | BLEND_MODE_DARK;
  u32 blackout;
  for(blackout=0;blackout<15;blackout+=1){
    REG_COLEY=BLEND_DEPTH(blackout);
    WaitForVsync();
  }
  REG_COLEY=BLEND_DEPTH(16);
  u32 palcnt;
  for(palcnt=0;palcnt<256;palcnt++){
    BGPAL[palcnt]=0;
  }
  REG_DISPCNT=0;
  ReturnPogoShell2();
}

void OpenMainMenu(void)
{
  SaveLastLine(LinePos);
  Status_HidePanel();
  Menu_Main();
  Status_All();
  if(Menu_Main_RequestReloadLinePos==True){
    LinePos=LoadLastLine();
    PageRedraw();
  }
  if(Menu_Main_RequestReloadFont==True){
    SaveLastLine(LinePos);
    ChangeFontPack(SRAM_Read32(&SRAM->FontPackIndex));
    LinePos=LoadLastLine();
    PageRedraw();
  }
  if(Menu_Main_RequestReloadLineSpace==True){
    SaveLastLine(LinePos);
    ChangeFontPack(SRAM_Read32(&SRAM->FontPackIndex));
    LinePos=LoadLastLine();
    PageRedraw();
  }
  if(Menu_Main_RequestReloadKeyRepeatTime==True){
    KeyRepeatTime=SRAM_Read32(&SRAM->KeyRepeatTime);
  }
}

int main(void)
{
//  SRAM_AllClear();
//  SRAM_SaveIWRAM(); while(1);
  
  memchkinit();
  memchk();
  
  *(u16*)0x4000204=0x0014; // set REG_WAITCNT 3/1 wait state
  
  Init_isAnkChar();
  
  sndbuf_Init();
  sndbuf_StartPCM();
  
  VBlank_Enabled=False;
  IRQTable[IRQIndex_DMA1]=(u32)sndbuf_DMA1_Handler;
  IRQTable[IRQIndex_VBlank]=(u32)IRQ_VBlank_Dummy;
  IRQ_Setup(IRQ_BIT_VBLANK|IRQ_BIT_DMA1,DSTAT_USE_VBLANK);
  
  BGInit();
  BG1Fade1();
  SetOAM();
  FontInit(&__iwram_overlay_lma);
  
  GetPogoShell2Args();
  PluginFilename=ps2Arg0;
  TextFilename=ps2Arg1;
  TextData=ps2ArgDataPtr;
  TextDataSize=ps2ArgDataSize;
  
  rtc_enable();
  
  SRAM_Start();
  SRAM_SelectState(TextFilename);
  
  ChangeFontPack(SRAM_Read32(&SRAM->FontPackIndex));
  KeyRepeatTime=SRAM_Read32(&SRAM->KeyRepeatTime);
  
  LinePos=0;
  PageRedraw();
  
  u32 NowKey;
  u32 LastKey;

  b8 KeyRepeat;
  KeyRepeat=False;
  
  if(SRAM_Read32(&SRAM_CurrentState->LastPosition)==0){
    Menu_BootDialog();
    }else{
    if(SRAM_Read32(&SRAM->LoadAutoState)==True){
      LinePos=LoadLastLine();
      if(LinePos!=0) PageRedraw();
      }else{
      Menu_AutoLoad();
      if(Menu_Main_RequestReloadLinePos==True){
        LinePos=LoadLastLine();
        if(LinePos!=0) PageRedraw();
      }
    }
  }
  
  Status_All();
  
  while((*KEYS)!=0x3ff);
  
  // Timer0 = ԌvfobOp Valiable
  // Timer1 = DirectSoundA
  // Timer2 = L[s[gp 250msLimit
  // Timer3 = ჌X|Xʕ\p 500msLimit
  
  b8 ChangeState=False;
  
  while(1){
    NowKey=(~*KEYS)&0x3ff;
    if(LastKey!=NowKey) KeyRepeat=False;
    LastKey=NowKey;
    
    if(NowKey==0){
      WaitForVsync();
      if(ChangeState==True){
        ChangeState=False;
        SaveLastLine(LinePos);
        Status_LinePos();
        REG_TM3CNT = 0;
        REG_TM3D=0;
        REG_TM3CNT = TM_FREQ_PER_256 | TM_ENABLE;
      }
      if(REG_TM3D>(65517/2)){
        REG_TM3CNT = 0;
        Status_BytePos();
        Status_Clock();
        u32 chkadr=memchk();
        if(chkadr!=0){
          uprintf("!! IWRAM CheckMemory broken at 0x%x",chkadr);
          FontData_BGCHRDrawText(BGCHR1,0,0,resprintf);
        }
      }
      }else{
      if(NowKey==(KEY_R|KEY_L)){
        _ReturnPogoShell2();
      }
      
      ChangeState=True;
      
      REG_TM2CNT = 0;
      REG_TM2D=0;
      REG_TM2CNT = TM_FREQ_PER_1024 | TM_ENABLE;
      
      if((NowKey==KEY_UP)||(NowKey==KEY_B)) TextScroll(-1,True);
      if((NowKey==KEY_DOWN)||(NowKey==KEY_A)) TextScroll(1,True);
      if(NowKey==KEY_LEFT){
        if(KeyRepeat==False) PlaySound(ISE_Page);
        PageScroll(-(s32)(LineCount-3));
      }
      if(NowKey==KEY_RIGHT){
        if(KeyRepeat==False) PlaySound(ISE_Page);
        PageScroll((s32)(LineCount-3));
      }
      
      if(NowKey==KEY_START){
        OpenMainMenu();
      }
      
      if(NowKey==KEY_SELECT){
        Status_HidePanel();
        Menu_HelpDialog();
        Status_All();
      }
      
      if(KeyRepeat==True){
//        WaitForVsync();
        }else{
        b8 LoopFlag;
        LoopFlag=True;
        while(LoopFlag==True){
          if((65117/4*KeyRepeatTime/1000)<REG_TM2D){ // 250ms wait
            KeyRepeat=True;
            LoopFlag=False;
          }
          if(NowKey!=((~*KEYS)&0x3ff)){ // ChangeKeyState
            KeyRepeat=False;
            LoopFlag=False;
          }
          if(LoopFlag==True) WaitForVsync();
        }
      }
      
    }
  }
  
}

