unit MainWin;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,_FontPack, ExtCtrls,_PicTools, StdCtrls, Buttons, Menus;

const EXEID='GBAMakeFont.exe for ps2txt ver 0.2';

type
  TFontInfo=record
    FontName:string;
    FontHeight:integer;
    EnabledAntialias:boolean;
    ANK_Alpha,ANK_Kana:boolean;
    SJIS_System,SJIS_noKanji,SJIS_KanjiLv1,SJIS_AlabiaNum:boolean;
    SJIS_KanjiLv2,SJIS_KanjiEtc:boolean;
  end;

const FontInfoCount=8;

const FontTypeCount=21;

type
  TMain = class(TForm)
    FontDlg: TFontDialog;
    FontSetGrp: TGroupBox;
    MainMenu1: TMainMenu;
    MM_File: TMenuItem;
    FontNameEdt: TLabeledEdit;
    Label1: TLabel;
    FontHeightLst: TComboBox;
    EnabledAntialiasChk: TCheckBox;
    ANK_AlphaChk: TCheckBox;
    ANK_KanaChk: TCheckBox;
    SJIS_noKanjiChk: TCheckBox;
    SJIS_KanjiLv1Chk: TCheckBox;
    SJIS_AlabiaNumChk: TCheckBox;
    SJIS_KanjiLv2Chk: TCheckBox;
    SJIS_KanjiEtcChk: TCheckBox;
    MM_FontSet: TMenuItem;
    MM_LoadTypicalFont: TMenuItem;
    PreviewImg: TImage;
    MM_CreateFontPack: TMenuItem;
    SJIS_SystemChk: TCheckBox;
    Button1: TButton;
    N1: TMenuItem;
    N2: TMenuItem;
    X1: TMenuItem;
    SaveDlg: TOpenDialog;
    DelDlg: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FontSetChangeClick(Sender: TObject);
    procedure MM_LoadTypicalFontClick(Sender: TObject);
    procedure MM_CreateFontPackClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure X1Click(Sender: TObject);
    procedure N1Click(Sender: TObject);
  private
    { Private 錾 }
  public
    { Public 錾 }
    FontInfo:array[0..FontInfoCount] of TFontInfo;
    CurrentFontInfo:integer;
    Changing:boolean;
    procedure onEncordCallback(pos,max:integer);
    procedure SetMenuLoadTypicalFont;
    procedure SetMenuFontSet;
    procedure LoadTypicalFont(var FontInfo:TFontInfo;FontType:integer);
    procedure LoadFontInfo;
    procedure SaveFontInfo;
    procedure DrawPreview;
    procedure FontInfoToFontPack(const FontInfo:TFontInfo;var FontPack:TFontPack);
    procedure MM_FontSetSubClick(Sender: TObject);
  end;

var
  Main: TMain;

implementation

{$R *.dfm}

procedure SeekID(var rfs:TFileStream;ID:string);
var
  cbuf:array[0..6] of char;
  i:integer;
  rbufcount:integer;
  rbuf:array[0..256*1024] of char;
begin
  for i:=0 to 6-1 do begin
    cbuf[i]:=ID[i+1];
  end;

  rbufcount:=rfs.Size;
  if rbufcount>256*1024 then rbufcount:=256*1024;

  rfs.Position:=0;
  rfs.ReadBuffer(rbuf[0],rbufcount);

  i:=0;

  while(i<(rbufcount-4)) do begin
    if rbuf[i+0]=cbuf[0] then begin
      if ((rbuf[i+1]=cbuf[1]) and (rbuf[i+2]=cbuf[2]) and (rbuf[i+3]=cbuf[3])) then begin
        rfs.Position:=i;
        exit;
      end;
    end;
    inc(i);
  end;

  rfs.Position:=rfs.Size;
end;

procedure DeleteMargeFPK(binfn:string);
var
  wfs:TFileStream;
begin
  wfs:=TFileStream.Create(binfn,fmOpenReadWrite);

  SeekID(wfs,'fpk2');
  if wfs.Size<>wfs.Position then wfs.Size:=wfs.Position;

  wfs.Free;
end;

procedure AppendMargeFPK(binfn:string);
var
  rfs,wfs:TFileStream;
  fpkcount:integer;
  fpkbuf:array of byte;
begin
  rfs:=TFileStream.Create(ChangeFileExt(binfn,'.fpk'),fmOpenRead);
  fpkcount:=rfs.Size;
  setlength(fpkbuf,fpkcount);
  rfs.ReadBuffer(fpkbuf[0],fpkcount);
  rfs.Free;

  wfs:=TFileStream.Create(binfn,fmOpenReadWrite);

  SeekID(wfs,'fpk2');
  if wfs.Size<>wfs.Position then wfs.Size:=wfs.Position;
  wfs.WriteBuffer(fpkbuf[0],fpkcount);

  wfs.Free;
end;

procedure TMain.SetMenuFontSet;
var
  cnt:integer;
  item:TMenuItem;
  ffn:string;
begin
  for cnt:=MM_FontSet.Count-1 downto 0 do begin
    MM_FontSet.Items[0].Free;
  end;

  for cnt:=1 to FontInfoCount-1 do begin
    if FontInfo[cnt].FontName='' then begin
      ffn:='Disabled.';
      end else begin
      ffn:=FontInfo[cnt].FontName+'_'+inttostr(FontInfo[cnt].FontHeight);
      if FontInfo[cnt].EnabledAntialias=True then ffn:=ffn+'Anti';
    end;
    item:=TMenuItem.Create(MM_FontSet);
    item.Tag:=cnt;
    item.Caption:='oN'+inttostr(cnt)+' '+ffn;
    item.OnClick:=MM_FontSetSubClick;
    MM_FontSet.Add(item);
  end;
end;

procedure TMain.SetMenuLoadTypicalFont;
var
  FontInfo:TFontInfo;
  cnt:integer;
  item:TMenuItem;
  ffn:string;
begin
  for cnt:=1 to FontTypeCount-1 do begin
    LoadTypicalFont(FontInfo,cnt);
    ffn:=FontInfo.FontName+'_';
    ffn:=ffn+inttostr(FontInfo.FontHeight);
    if FontInfo.EnabledAntialias=True then ffn:=ffn+'Anti';
    item:=TMenuItem.Create(MM_LoadTypicalFont);
    item.Tag:=cnt;
    if cnt=1 then begin
      item.Caption:=inttostr(cnt)+' Disabled.';
      end else begin
      item.Caption:=inttostr(cnt)+' '+ffn;
    end;
    item.OnClick:=MM_LoadTypicalFontClick;
    MM_LoadTypicalFont.Add(item);
  end;
end;

procedure TMain.FormCreate(Sender: TObject);
begin
  Main.Caption:=EXEID;
  
  SetMenuLoadTypicalFont;

  LoadTypicalFont(FontInfo[0],0);
  LoadTypicalFont(FontInfo[1],7);
  LoadTypicalFont(FontInfo[2],3);
  LoadTypicalFont(FontInfo[3],-1);
  LoadTypicalFont(FontInfo[4],-1);
  LoadTypicalFont(FontInfo[5],-1);
  LoadTypicalFont(FontInfo[6],-1);
  LoadTypicalFont(FontInfo[7],-1);

  SetMenuFontSet;
  CurrentFontInfo:=1;
  LoadFontInfo;
end;

procedure TMain.LoadTypicalFont(var FontInfo:TFontInfo;FontType:integer);
begin
  if FontType=-1 then begin
    with FontInfo do begin
      FontName:='';
      FontHeight:=10;
      EnabledAntialias:=True;
      ANK_Alpha:=True;
      ANK_Kana:=True;
      SJIS_System:=False;
      SJIS_noKanji:=True;
      SJIS_KanjiLv1:=True;
      SJIS_AlabiaNum:=True;
      SJIS_KanjiLv2:=True;
      SJIS_KanjiEtc:=True;
    end;
    exit;
  end;

  if FontType=0 then begin
    with FontInfo do begin
      FontName:='lr SVbN';
      FontHeight:=10;
      EnabledAntialias:=False;
      ANK_Alpha:=True;
      ANK_Kana:=True;
      SJIS_System:=True;
      SJIS_noKanji:=False;
      SJIS_KanjiLv1:=False;
      SJIS_AlabiaNum:=False;
      SJIS_KanjiLv2:=False;
      SJIS_KanjiEtc:=False;
    end;
    exit;
  end;

  with FontInfo do begin
    ANK_Alpha:=True;
    ANK_Kana:=True;
    SJIS_System:=False;
    SJIS_noKanji:=True;
    SJIS_KanjiLv1:=True;
    SJIS_AlabiaNum:=True;
    SJIS_KanjiLv2:=True;
    SJIS_KanjiEtc:=True;
  end;

  with FontInfo do begin
    case FontType of
      1: begin
        FontName:='';
        FontHeight:=10;
        EnabledAntialias:=True;
      end;
      2: begin
        FontName:='lr SVbN';
        FontHeight:=8;
        EnabledAntialias:=False;
      end;
      3: begin
        FontName:='lr SVbN';
        FontHeight:=10;
        EnabledAntialias:=False;
      end;
      4: begin
        FontName:='lr SVbN';
        FontHeight:=12;
        EnabledAntialias:=False;
      end;
      5: begin
        FontName:='lr SVbN';
        FontHeight:=16;
        EnabledAntialias:=False;
      end;
      6: begin
        FontName:='lr ';
        FontHeight:=8;
        EnabledAntialias:=True;
      end;
      7: begin
        FontName:='lr ';
        FontHeight:=10;
        EnabledAntialias:=True;
      end;
      8: begin
        FontName:='lr ';
        FontHeight:=12;
        EnabledAntialias:=True;
      end;
      9: begin
        FontName:='lr ';
        FontHeight:=16;
        EnabledAntialias:=True;
      end;
      10: begin
        FontName:='Terminal';
        FontHeight:=8;
        EnabledAntialias:=False;
      end;
      11: begin
        FontName:='Terminal';
        FontHeight:=13;
        EnabledAntialias:=False;
      end;
      12: begin
        FontName:='Terminal';
        FontHeight:=19;
        EnabledAntialias:=False;
      end;
      13: begin
        FontName:='Terminal';
        FontHeight:=23;
        EnabledAntialias:=False;
      end;
      14: begin
        FontName:='FixedSys';
        FontHeight:=19;
        EnabledAntialias:=False;
      end;
      15: begin
        FontName:='Small Fonts';
        FontHeight:=8;
        EnabledAntialias:=False;
      end;
      16: begin
        FontName:='ce܂W9';
        FontHeight:=8;
        EnabledAntialias:=True;
      end;
      17: begin
        FontName:='ce܂W9';
        FontHeight:=10;
        EnabledAntialias:=True;
      end;
      18: begin
        FontName:='ce܂W9';
        FontHeight:=12;
        EnabledAntialias:=True;
      end;
      19: begin
        FontName:='ce܂W9';
        FontHeight:=16;
        EnabledAntialias:=True;
      end;
      20: begin
        FontName:='ce܂W9';
        FontHeight:=16;
        EnabledAntialias:=False;
      end;
    end;
  end;

end;

procedure TMain.LoadFontInfo;
begin
  Changing:=True;

  FontSetGrp.Caption:='tHgZbg oN'+inttostr(CurrentFontInfo);

  with FontInfo[CurrentFontInfo] do begin
    FontNameEdt.Text:=FontName;
    FontHeightLst.ItemIndex:=FontHeight;
    EnabledAntialiasChk.Checked:=EnabledAntialias;
    ANK_AlphaChk.Checked:=ANK_Alpha;
    ANK_KanaChk.Checked:=ANK_Kana;
    SJIS_SystemChk.Checked:=SJIS_System;
    SJIS_noKanjiChk.Checked:=SJIS_noKanji;
    SJIS_KanjiLv1Chk.Checked:=SJIS_KanjiLv1;
    SJIS_AlabiaNumChk.Checked:=SJIS_AlabiaNum;
    SJIS_KanjiLv2Chk.Checked:=SJIS_KanjiLv2;
    SJIS_KanjiEtcChk.Checked:=SJIS_KanjiEtc;
  end;

  Changing:=False;

  DrawPreview;
  SetMenuFontSet;
end;

procedure TMain.SaveFontInfo;
begin
  with FontInfo[CurrentFontInfo] do begin
    FontName:=FontNameEdt.Text;
    FontHeight:=FontHeightLst.ItemIndex;
    EnabledAntialias:=EnabledAntialiasChk.Checked;
    ANK_Alpha:=ANK_AlphaChk.Checked;
    ANK_Kana:=ANK_KanaChk.Checked;
    SJIS_System:=SJIS_SystemChk.Checked;
    SJIS_noKanji:=SJIS_noKanjiChk.Checked;
    SJIS_KanjiLv1:=SJIS_KanjiLv1Chk.Checked;
    SJIS_AlabiaNum:=SJIS_AlabiaNumChk.Checked;
    SJIS_KanjiLv2:=SJIS_KanjiLv2Chk.Checked;
    SJIS_KanjiEtc:=SJIS_KanjiEtcChk.Checked;
  end;
  SetMenuFontSet;
end;

procedure TMain.FontInfoToFontPack(const FontInfo:TFontInfo;var FontPack:TFontPack);
begin
  with FontPack do begin
    FontName:=FontInfo.FontName;
    FontHeight:=FontInfo.FontHeight;
    EnabledAntialias:=FontInfo.EnabledAntialias;
    with FontSet do begin
      ANK_Alpha:=FontInfo.ANK_Alpha;
      ANK_Kana:=FontInfo.ANK_Kana;
      SJIS_System:=FontInfo.SJIS_System;
      SJIS_noKanji:=FontInfo.SJIS_noKanji;
      SJIS_KanjiLv1:=FontInfo.SJIS_KanjiLv1;
      SJIS_AlabiaNum:=FontInfo.SJIS_AlabiaNum;
      SJIS_KanjiLv2:=FontInfo.SJIS_KanjiLv2;
      SJIS_KanjiEtc:=FontInfo.SJIS_KanjiEtc;
    end;
    FontWidth:=0;
    EncordCallback:=onEncordCallback;
  end;
end;

procedure TMain.DrawPreview;
var
  FontPack:TFontPack;
  str:string;
begin
  if Changing=True then exit;

  if FontInfo[CurrentFontInfo].FontName='' then begin
    MakeBlankImg(PreviewImg,pf24bit);
    exit;
  end;
  
  FontPack:=TFontPack.Create;
  FontInfoToFontPack(FontInfo[CurrentFontInfo],FontPack);

  with FontPack.DrawMode do begin
    SetRGBPack(TextColor,$000000);
    Transparent:=False;
    SetRGBPack(BGColor,$ffffff);
  end;

  str:='';
  with FontPack.FontSet do begin
    if ANK_Alpha then str:=str+'abcdABCD'+chr($09);
    if ANK_Kana then str:=str+'';
    if SJIS_noKanji then str:=str+'';
    if SJIS_KanjiLv1 then str:=str+'';
    if SJIS_AlabiaNum then str:=str+'@ABCD';
    if SJIS_KanjiLv2 then str:=str+'';
    if SJIS_KanjiEtc then str:=str+'\]^_`';
  end;

  MakeBlankImg(PreviewImg,pf24bit);
  FontPack.DrawDecordStr(PreviewImg.Picture.Bitmap,0,0,str,False,False,False);
  PreviewImg.Refresh;

  FontPack.Free;
end;

procedure TMain.FontSetChangeClick(Sender: TObject);
begin
  if Changing=True then exit;
  SaveFontInfo;
  DrawPreview;
end;

procedure TMain.MM_LoadTypicalFontClick(Sender: TObject);
var
  t:integer;
begin
  t:=(Sender as TMenuItem).Tag;
  if t<=0 then exit;
  LoadTypicalFont(FontInfo[CurrentFontInfo],t);
  LoadFontInfo;
end;

procedure TMain.MM_FontSetSubClick(Sender: TObject);
var
  t:integer;
begin
  t:=(Sender as TMenuItem).Tag;
  if t=-1 then exit;
  CurrentFontInfo:=t;
  LoadFontInfo;
end;

var
  CurFPKFN:string;

type
  Ttmp=record
    FontInfo:TFontInfo;
    Filename:string;
  end;

type
  THeader=record
    ID:string;
    Size:dword;
    fpkFilenameStr:array[0..FontInfoCount] of string;
    fpkFilename:array[0..FontInfoCount,0..32] of char;
    fpkFileOffset:array[0..FontInfoCount] of dword;
    fpkFileSize:array[0..FontInfoCount] of dword;
  end;

procedure TMain.MM_CreateFontPackClick(Sender: TObject);
var
  Header:THeader;
  fpkcnt:integer;
  ffn:string;
  sc:integer;
  rfs,wfs:TFileStream;
  buf:array of byte;
  procedure createfpk(FontInfo:TFontInfo;fpkFilename:string);
  var
    FontPack:TFontPack;
  begin
    CurFPKFN:=fpkFilename;
    FontPack:=TFontPack.Create;
    FontInfoToFontPack(FontInfo,FontPack);

    if FontPack.FullAutoEncord=True then begin
      FontPack.SaveToFile(fpkFilename);
      end else begin
      ShowMessage('tHgpbP[W̍쐬Ɏs܂B');
    end;
    FontPack.Free;
  end;
  procedure wfspadding;
  var
    d:dword;
    s:integer;
  begin
    s:=4-(wfs.Position mod 4);
    if s=4 then exit;
    d:=0;
    wfs.Write(d,s);
  end;
begin
  if SaveDlg.Execute=False then exit;

  with Header do begin
    ID:='fpk2';
    Size:=0;
    for fpkcnt:=0 to FontInfoCount-1 do begin
      fpkFilenameStr[fpkcnt]:='';
      for sc:=0 to 31 do begin
        fpkFilename[fpkcnt][sc]:=char($00);
      end;
      fpkFileOffset[fpkcnt]:=0;
      fpkFileSize[fpkcnt]:=0;
    end;
  end;

  for fpkcnt:=0 to FontInfoCount-1 do begin
    if FontInfo[fpkcnt].FontName<>'' then begin
      if fpkcnt=0 then begin
        ffn:='SystenFont';
        end else begin
        ffn:=FontInfo[fpkcnt].FontName+'_'+inttostr(FontInfo[fpkcnt].FontHeight);
        if FontInfo[fpkcnt].EnabledAntialias=True then ffn:=ffn+'Anti';
      end;
      ffn:=ffn+'.fpk';
      Header.fpkFilenameStr[fpkcnt]:=ffn;
      createfpk(FontInfo[fpkcnt],ffn);
      for sc:=0 to 31 do begin
        if sc<Length(ffn) then begin
          Header.fpkFilename[fpkcnt][sc]:=ffn[sc+1];
          end else begin
          Header.fpkFilename[fpkcnt][sc]:=char($00);
        end;
      end;
    end;
  end;

  wfs:=TFileStream.Create(ChangeFileExt(SaveDlg.FileName,'.fpk'),fmCreate);

  wfs.Position:=0;
  with Header do begin
    wfs.Write(ID[1],4);
    wfs.Write(Size,4);
    for fpkcnt:=0 to FontInfoCount-1 do begin
      wfs.Write(fpkFilename[fpkcnt,0],32);
      wfs.Write(fpkFileOffset[fpkcnt],4);
      wfs.Write(fpkFileSize[fpkcnt],4);
    end;
    wfspadding;
  end;

  with Header do begin
    for fpkcnt:=0 to FontInfoCount-1 do begin
      if FontInfo[fpkcnt].FontName<>'' then begin
        rfs:=TFileStream.Create(Header.fpkFilenameStr[fpkcnt],fmOpenRead);
        fpkFileOffset[fpkcnt]:=wfs.Position;
        fpkFileSize[fpkcnt]:=rfs.Size;
        setlength(buf,fpkFileSize[fpkcnt]);
        rfs.Read(buf[0],fpkFileSize[fpkcnt]);
        rfs.Free;
        wfs.Write(buf[0],fpkFileSize[fpkcnt]);
        wfspadding;
        DeleteFile(Header.fpkFilenameStr[fpkcnt]);
      end;
    end;
  end;

  Header.Size:=wfs.Size;

  wfs.Position:=0;
  with Header do begin
    wfs.Write(ID[1],4);
    wfs.Write(Size,4);
    for fpkcnt:=0 to FontInfoCount-1 do begin
      wfs.Write(fpkFilename[fpkcnt,0],32);
      wfs.Write(fpkFileOffset[fpkcnt],4);
      wfs.Write(fpkFileSize[fpkcnt],4);
    end;
  end;

  wfs.Free;

  Main.Caption:=EXEID;

  AppendMargeFPK(SaveDlg.FileName);

  ShowMessage('tHgZbǧ܂B'+CHR(13)+CHR(10)+'txt.binvOC{̂łB');
end;

procedure TMain.Button1Click(Sender: TObject);
begin
  with FontDlg.Font do begin
    Name:=FontInfo[CurrentFontInfo].FontName;
    Height:=-FontInfo[CurrentFontInfo].FontHeight;
    Charset:=SHIFTJIS_CHARSET;
    Pitch:=fpFixed;
  end;

  if FontDlg.Execute=True then begin
    with FontDlg.Font do begin
      FontInfo[CurrentFontInfo].FontName:=Name;
      FontInfo[CurrentFontInfo].FontHeight:=-Height;
    end;
    LoadFontInfo;
  end;
end;

procedure TMain.onEncordCallback(pos,max:integer);
begin
  if (pos mod 128)=0 then begin
    Main.Caption:=CurFPKFN+' '+format('%d/%d',[pos,max]);
  end;
end;

procedure TMain.X1Click(Sender: TObject);
begin
  Main.Close;
end;

procedure TMain.N1Click(Sender: TObject);
begin
  if DelDlg.Execute=False then exit;

  DeleteMargeFPK(DelDlg.FileName);

  ShowMessage('txt.bintHgZbg܂B'+CHR(13)+CHR(10)+'txt.bin͂̂܂܂ł͋N܂B');
end;

end.
